@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -45,11 +45,9 @@
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     ivar          variable number
 !> \param[in]     isou          local variable number (7 here)
-!> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
 !>                               (see \ref cs_user_mass_source_terms)
@@ -58,27 +56,25 @@
 !>                                 only for iturb=31
 !> \param[in]     gradro        work array for grad rom
 !>                              (without rho volume) only for iturb=30
-!> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
 !>                               source terms or mass rate
 !>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     viscf         visc*surface/dist at internal faces
 !> \param[in]     viscb         visc*surface/dist at edge faces
-!> \param[in]     tslage        explicit source terms for the Lagrangian module
 !> \param[in]     tslagi        implicit source terms for the Lagrangian module
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
 !_______________________________________________________________________________
 
 subroutine resssg &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+ ( nvar   , nscal  , ncesmp ,                                     &
    ivar   , isou   ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
    gradv  , gradro ,                                              &
-   ckupdc , smacel ,                                              &
+   smacel ,                                                       &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -102,6 +98,7 @@ use field_operator
 use cs_f_interfaces
 use rotation
 use turbomachinery
+use cs_c_bindings
 
 !===============================================================================
 
@@ -110,36 +107,38 @@ implicit none
 ! Arguments
 
 integer          nvar   , nscal
-integer          ncepdp , ncesmp
+integer          ncesmp
 integer          ivar   , isou
 
-integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
 double precision gradv(3, 3, ncelet)
 double precision gradro(3,ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslage(ncelet),tslagi(ncelet)
+double precision tslagi(ncelet)
 double precision smbr(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
 integer          iel
-integer          ii    , jj    , kk    , iiun  , iii   , jjj
+integer          ii    , jj    , kk    , iii   , jjj
 integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
-integer          nswrsp, ircflp, ischcp, isstpp, iescap
-integer          st_prv_id
+integer          imrgrp, nswrsp, ircflp, ischcp, isstpp, iescap
+integer          st_prv_id, comp_id
 integer          iprev , inc, iccocg, ll
 integer          imucpp, idftnp, iswdyp
 integer          ivar_r(3,3)
 integer          icvflb
+integer          init
 integer          ivoid(1)
+integer          key_t_ext_id
+integer          iroext
 
-double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
+double precision blencp, epsilp, epsrgp, climgp, relaxp
 double precision epsrsp
 double precision trprod, trrij , deltij
 double precision tuexpr, thets , thetv , thetp1
@@ -152,6 +151,7 @@ double precision pij, phiij1, phiij2, epsij
 double precision phiijw, epsijw
 double precision ccorio
 double precision rctse
+double precision normp
 
 double precision rvoid(1)
 
@@ -166,24 +166,31 @@ double precision, allocatable, dimension(:) :: weighb
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: crom, cromo
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
-double precision, dimension(:,:), pointer :: visten
+double precision, dimension(:,:), pointer :: visten, lagr_st_rij
 double precision, dimension(:), pointer :: cvara_ep, cvar_al
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
 double precision, dimension(:), pointer :: cvara_r12, cvara_r13, cvara_r23
 double precision, dimension(:), pointer :: cvar_var, cvara_var
-double precision, dimension(:), pointer :: viscl, c_st_prv
+double precision, dimension(:), pointer :: viscl, c_st_prv, visct
 
 type pmapper_double_r1
    double precision, dimension(:),  pointer :: p !< rank 1 array pointer
 end type pmapper_double_r1
 type(pmapper_double_r1), allocatable, dimension(:,:) :: cvara_r
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
 ! 1. Initialization
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
+call field_get_key_int(icrom, key_t_ext_id, iroext)
+
 ! Allocate work arrays
 allocate(w1(ncelet), w2(ncelet))
 allocate(dpvar(ncelet))
@@ -195,13 +202,16 @@ allocate(weighb(nfabor))
 iii = 0
 jjj = 0
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
 if (iturb.ne.31) call field_get_val_s(ivarfl(ial), cvar_al)
@@ -232,7 +242,7 @@ endif
 
 !     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
 call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
 if (st_prv_id .ge. 0) then
@@ -290,12 +300,7 @@ endif
 ! 2. User source terms
 !===============================================================================
 
-call cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   smbr   , rovsdt )
+call user_source_terms(ivarfl(ivar), smbr, rovsdt)
 
 !     If we extrapolate the source terms
 if (st_prv_id.ge.0) then
@@ -324,8 +329,15 @@ endif
 
  !     2nd order is not taken into account
  if (iilagr.eq.2 .and. ltsdyn.eq.1) then
+   call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+   comp_id = isou
+   if (isou .eq. 5) then
+     comp_id = 6
+   else if (isou .eq.6) then
+     comp_id = 5
+   endif
    do iel = 1,ncel
-     smbr(iel)   = smbr(iel)   + tslage(iel)
+     smbr(iel)   = smbr(iel)   + lagr_st_rij(comp_id,iel)
      rovsdt(iel) = rovsdt(iel) + max(-tslagi(iel),zero)
    enddo
  endif
@@ -336,15 +348,10 @@ endif
 
 if (ncesmp.gt.0) then
 
-  !       Integer equal to 1 (for navsto: nb of sur-iter)
-  iiun = 1
-
-  ! We increment smbr with -Gamma.var_prev and rovsdr with Gamma (*theta)
-  call catsma &
- ( ncelet , ncel   , ncesmp , iiun   , isto2t , thetv  ,          &
-   icetsm , itypsm(:,ivar)  ,                                     &
-   cell_f_vol , cvara_var       , smacel(:,ivar)   , smacel(:,ipr) ,  &
-   smbr   ,  rovsdt , w1 )
+  ! We increment smbr with -Gamma.var_prev and rovsdr with Gamma
+  call catsma(ncesmp, 1, icetsm, itypsm(:,ivar),                     &
+              cell_f_vol, cvara_var, smacel(:,ivar), smacel(:,ipr),  &
+              smbr,  rovsdt, w1)
 
   ! If we extrapolate the source terms we put Gamma Pinj in the previous st
   if (st_prv_id.ge.0) then
@@ -366,9 +373,9 @@ endif
 
 ! ---> Added in the matrix diagonal
 
-do iel=1,ncel
+do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel)                                       &
-            + istat(ivar)*(crom(iel)/dt(iel))*cell_f_vol(iel)
+            + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 
@@ -412,9 +419,7 @@ if (iturb.eq.32) then
   inc    = 1
   iccocg = 1
 
-  call field_gradient_scalar(ivarfl(ial), iprev, imrgra, inc,     &
-                             iccocg,                              &
-                             grad)
+  call field_gradient_scalar(ivarfl(ial), iprev, 0, inc, iccocg, grad)
 
 endif
 
@@ -717,7 +722,7 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
@@ -730,7 +735,7 @@ if (idften(ivar).eq.6) then
     viscce(6,iel) = visten(6,iel)
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
  ( viscce , iwarnp ,             &
@@ -741,9 +746,8 @@ if (idften(ivar).eq.6) then
 else
 
   do iel = 1, ncel
-    trrij = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
-    rctse = crom(iel) * csrij * trrij**2 / cvara_ep(iel)
-    w1(iel) = viscl(iel) + idifft(ivar)*rctse
+    rctse = csrij * visct(iel) / cmu
+    w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
   call viscfa                    &
@@ -764,29 +768,31 @@ if (st_prv_id.ge.0) then
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
+ndircp = vcopt%ndircl
+imrgrp = vcopt%imrgra
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
+init   = 1
 
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
@@ -794,16 +800,16 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
    cvara_var       , cvara_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
-   viscf  , viscb  , viscce , viscf  , viscb  , viscce ,          &
+   viscf  , viscb  , viscf  , viscb  , viscce ,                   &
    weighf , weighb ,                                              &
    icvflb , ivoid  ,                                              &
    rovsdt , smbr   , cvar_var        , dpvar  ,                   &
@@ -819,16 +825,8 @@ deallocate(weighf, weighb)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,'           Resolution pour la variable ',A8,/)
-
-#else
-
  1000 format(/,'           Solving variable ',A8           ,/)
 
-#endif
-
 !----
 ! End
 !----
